/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
*
* Copyright 2013 - 2020, nymea GmbH
* Contact: contact@nymea.io
*
* This file is part of maveo-scanner.
* This project including source code and documentation is protected by
* copyright law, and remains the property of nymea GmbH. All rights, including
* reproduction, publication, editing and translation, are reserved. The use of
* this project is subject to the terms of a license agreement to be concluded
* with nymea GmbH in accordance with the terms of use of nymea GmbH, available
* under https://nymea.io/license
*
* For any further details and any questions please contact us under
* contact@nymea.io or see our FAQ/Licensing Information on
* https://nymea.io/license/faq
*
* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

#ifndef BOOTLOADERUTILS_H
#define BOOTLOADERUTILS_H

#include <QObject>

#include "../maveoscannerutils.h"

class LIBMAVEO_SCANNER_EXPORT BootloaderUtils
{
    Q_GADGET
public:
    enum ChecksumType {
        ChecksumTypeSum = 0x00,
        ChecksumTypeCrc = 0x01,
        ChecksumTypeUnknown = 0xff
    };
    Q_ENUM(ChecksumType)

    enum BootloaderCommand {
        BootloaderCommandVerifyChecksum = 0x31,
        BootloaderCommandGetFlashSize = 0x32,
        BootloaderCommandGetApplicationStatus = 0x33,
        BootloaderCommandEraseRow = 0x34,
        BootloaderCommandSyncBootloader = 0x35,
        BootloaderCommandSetActiveApplication = 0x36,
        BootloaderCommandSendData = 0x37,
        BootloaderCommandEnterBootloader = 0x38,
        BootloaderCommandProgrammRow = 0x39,
        BootloaderCommandVerifyRow = 0x3A,
        BootloaderCommandExitBootloader = 0x3B
    };
    Q_ENUM(BootloaderCommand)

    enum BootloaderError {
        BootloaderErrorNoError = 0x00,
        BootloaderErrorVerificationFailed = 0x02,
        BootloaderErrorInvalidLength = 0x03,
        BootloaderErrorInvalidData = 0x04,
        BootloaderErrorInvalidCommand = 0x05,
        BootloaderErrorUnexpectedDevice = 0x06,
        BootloaderErrorBootloaderVersion = 0x07,
        BootloaderErrorChecksum = 0x08,
        BootloaderErrorInvalidIdArray = 0x09,
        BootloaderErrorInvalidRowNumber = 0x0a,
        BootloaderErrorInvalidApplication = 0x0c,
        BootloaderErrorApplicationActive = 0x0d,
        BootloaderErrorUnknownError = 0x0f
    };
    Q_ENUM(BootloaderError)


    explicit BootloaderUtils() = default;
    static quint16 calculateChecksum(const QByteArray &data, ChecksumType checksumType = ChecksumTypeSum);
    static QByteArray readFirmwareAsciiData(const QByteArray &data);

};

#endif // BOOTLOADERUTILS_H
