/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
*
* Copyright 2013 - 2020, nymea GmbH
* Contact: contact@nymea.io
*
* This file is part of maveo-scanner.
* This project including source code and documentation is protected by
* copyright law, and remains the property of nymea GmbH. All rights, including
* reproduction, publication, editing and translation, are reserved. The use of
* this project is subject to the terms of a license agreement to be concluded
* with nymea GmbH in accordance with the terms of use of nymea GmbH, available
* under https://nymea.io/license
*
* For any further details and any questions please contact us under
* contact@nymea.io or see our FAQ/Licensing Information on
* https://nymea.io/license/faq
*
* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

#ifndef FIRMWAREDOWNLOADER_H
#define FIRMWAREDOWNLOADER_H

#include <QObject>
#include <QDateTime>
#include <QNetworkReply>
#include <QNetworkAccessManager>

#include "firmwarefile.h"
#include "bootloaderutils.h"

class LIBMAVEO_SCANNER_EXPORT FirmwareDownloader : public QObject
{
    Q_OBJECT
    Q_PROPERTY(QString updateFileName READ updateFileName NOTIFY updateFileNameChanged)
    Q_PROPERTY(QString versionString READ versionString NOTIFY versionStringChanged)
    Q_PROPERTY(QString hardwareName READ hardwareName NOTIFY hardwareNameChanged)
    Q_PROPERTY(QDateTime releaseDateTime READ releaseDateTime NOTIFY releaseDateTimeChanged)
    Q_PROPERTY(FirmwareData firmware READ firmware NOTIFY firmwareChanged)
    Q_PROPERTY(bool preReleaseEnabled READ preReleaseEnabled WRITE setPreReleaseEnabled NOTIFY preReleaseEnabledChanged)

public:
    explicit FirmwareDownloader(QObject *parent = nullptr);

    bool updateDataAvailable() const;
    bool updateInformationAvailable() const;

    bool preReleaseEnabled() const;
    void setPreReleaseEnabled(bool preReleaseEnabled);

    QString updateFileName() const;
    QString versionString() const;
    QString hardwareName() const;
    QDateTime releaseDateTime() const;
    int majorVersion() const;
    int minorVersion() const;
    int patchVersion() const;

    FirmwareData firmware() const;

private:
    QNetworkAccessManager *m_networkAccessManager = nullptr;

    QString m_releaseUrlBase = "https://downloads.nymea.io/firmware/maveo-scanner";

    bool m_updateInformationAvailable = false;
    bool m_updateDataAvailable = false;
    bool m_preReleaseEnabled = false;

    QString m_versionString;
    QString m_hardwareName;
    QString m_updateFileName;
    QString m_updateFileChecksum;
    QDateTime m_releaseDateTime;
    int m_majorVersion = 0;
    int m_minorVersion = 0;
    int m_patchVersion = 0;
    FirmwareData m_firmware;

    void setUpdateInformationAvailable(bool available);
    void setUpdateDataAvailable(bool available);

    void setVersionString(const QString &versionString);
    void setHardwareName(const QString &hardwareName);
    void setUpdateFileName(const QString &updateFileName);
    void setReleaseDateTime(const QDateTime &releaseDataTime);
    void setFirmware(const FirmwareData &firmware);

    void reset();

signals:
    void versionStringChanged(const QString &versionString);
    void hardwareNameChanged(const QString &hardwareName);
    void updateFileNameChanged(const QString &updateFileName);
    void releaseDateTimeChanged(const QDateTime &releaseDateTime);
    void firmwareChanged(const FirmwareData &firmware);

    void firmwareUpdateFinished();
    void firmwareDownloadFinished();
    void preReleaseEnabledChanged(bool preReleaseEnabled);

public slots:
    void checkNewFirmware();
    void downloadFirmware();

};

#endif // FIRMWAREDOWNLOADER_H
