/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
*
* Copyright 2013 - 2020, nymea GmbH
* Contact: contact@nymea.io
*
* This file is part of maveo-scanner.
* This project including source code and documentation is protected by
* copyright law, and remains the property of nymea GmbH. All rights, including
* reproduction, publication, editing and translation, are reserved. The use of
* this project is subject to the terms of a license agreement to be concluded
* with nymea GmbH in accordance with the terms of use of nymea GmbH, available
* under https://nymea.io/license
*
* For any further details and any questions please contact us under
* contact@nymea.io or see our FAQ/Licensing Information on
* https://nymea.io/license/faq
*
* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

#ifndef FIRMWAREDATA_H
#define FIRMWAREDATA_H

#include <QFile>
#include <QFileInfo>
#include <QByteArray>
#include <QDataStream>

#include "bootloaderutils.h"

class LIBMAVEO_SCANNER_EXPORT FirmwareDataRow
{    
public:
    FirmwareDataRow(const QByteArray &data);

    QByteArray data() const;
    QByteArray rowData() const;
    quint8 arrayId() const;
    quint16 rowNumber() const;
    quint16 rowSize() const;
    quint8 checksum() const;
    quint8 verifyChecksum() const;
    bool isValid() const;

private:
    QByteArray m_data;
    QByteArray m_rowData;

    quint8 m_arrayId = 0;
    quint16 m_rowNumber = 0;
    quint16 m_rowSize = 0;
    quint8 m_checksum = 0;
    quint8 m_verifyChecksum = 0;

    bool m_valid = false;
};

QDebug operator<<(QDebug debug, const FirmwareDataRow &dataRow);

class LIBMAVEO_SCANNER_EXPORT FirmwareData
{
    Q_GADGET
    Q_PROPERTY(int siliconId READ siliconId CONSTANT)
    Q_PROPERTY(int siliconRev READ siliconRev CONSTANT)

public:
    FirmwareData();
    FirmwareData(const QByteArray &data);
    FirmwareData(const QString &fileName);

    QString name() const;
    void setName(const QString &name);

    QString applicationVersion() const;
    void setApplicationVersion(const QString &applicationVersion);

    void setData(const QByteArray &data);
    void setFileName(const QString &fileName);

    int siliconId() const;
    int siliconRev() const;
    BootloaderUtils::ChecksumType checksumType() const;
    QList<FirmwareDataRow> rows() const;

    void clear();
    bool isValid() const;

private:
    QString m_name;
    QString m_applicationVersion;

    QString m_fileName;
    QByteArray m_data;

    QList<FirmwareDataRow> m_rows;
    quint32 m_siliconId = 0;
    quint8 m_siliconRev = 0xff;
    BootloaderUtils::ChecksumType m_checksumType = BootloaderUtils::ChecksumTypeUnknown;

    bool m_valid = false;

    void processData();
};

QDebug operator<<(QDebug debug, const FirmwareData &firmwareData);


#endif // FIRMWAREDATA_H
