/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
*
* Copyright 2013 - 2020, nymea GmbH
* Contact: contact@nymea.io
*
* This file is part of maveo-scanner.
* This project including source code and documentation is protected by
* copyright law, and remains the property of nymea GmbH. All rights, including
* reproduction, publication, editing and translation, are reserved. The use of
* this project is subject to the terms of a license agreement to be concluded
* with nymea GmbH in accordance with the terms of use of nymea GmbH, available
* under https://nymea.io/license
*
* For any further details and any questions please contact us under
* contact@nymea.io or see our FAQ/Licensing Information on
* https://nymea.io/license/faq
*
* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

#ifndef HONEYWELLREPLY_H
#define HONEYWELLREPLY_H

#include <QObject>
#include <QTimer>

#include "maveoscannerutils.h"
#include "honeywellscanner.h"

class LIBMAVEO_SCANNER_EXPORT HoneywellReply : public QObject
{
    Q_OBJECT

    friend class HoneywellScanner;

public:
    // Request
    QString command() const;
    char target();
    QByteArray requestData() const;

    // Response
    HoneywellScanner::Response response() const;
    QByteArray responseData() const;

    bool timedOut() const;
    bool aborted() const;

private:
    explicit HoneywellReply(QObject *parent = nullptr);

    QTimer m_timer;
    bool m_timedOut = false;
    int m_timeoutInterval = 10000;

    bool m_aborted = false;

    QString m_command;
    char m_target = '0';
    QByteArray m_requestData;

    HoneywellScanner::Response m_response = HoneywellScanner::ResponseUnknown;
    QByteArray m_responseData;

    void startWait();
    void setFinished();
    void abort();

signals:
    void finished();

private slots:
    void onTimeout();

};

#endif // HONEYWELLREPLY_H
