/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
*
* Copyright 2013 - 2020, nymea GmbH
* Contact: contact@nymea.io
*
* This file is part of maveo-scanner.
* This project including source code and documentation is protected by
* copyright law, and remains the property of nymea GmbH. All rights, including
* reproduction, publication, editing and translation, are reserved. The use of
* this project is subject to the terms of a license agreement to be concluded
* with nymea GmbH in accordance with the terms of use of nymea GmbH, available
* under https://nymea.io/license
*
* For any further details and any questions please contact us under
* contact@nymea.io or see our FAQ/Licensing Information on
* https://nymea.io/license/faq
*
* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

#ifndef MAVEOSCANNERDEVICEWATCHER_H
#define MAVEOSCANNERDEVICEWATCHER_H

#include <QObject>

#ifdef __linux__
#include <QSocketNotifier>
#include <libudev.h>
#endif

#ifdef WINDOWS
#include <QTimer>
#include <QSerialPort>
#include <QSerialPortInfo>
#endif

#include "maveoscannerutils.h"

class LIBMAVEO_SCANNER_EXPORT MaveoScannerDeviceWatcher : public QObject
{
    Q_OBJECT
public:

    typedef struct DeviceInfo {
        QString systemLocation;
        QString serialNumber;
    } DeviceInfo;

    explicit MaveoScannerDeviceWatcher(QObject *parent = nullptr);
    ~MaveoScannerDeviceWatcher();

    QList<DeviceInfo> deviceInfos() const;

private:
    QHash<QString, DeviceInfo> m_deviceInfos;

#ifdef __linux__
    struct udev *m_udev = nullptr;
    struct udev_monitor *m_monitor = nullptr;
    QSocketNotifier *m_notifier = nullptr;
#endif

#ifdef WINDOWS
    QTimer *m_timer = nullptr;
    void synchSerialPorts();
#endif

signals:
    void deviceInfoAdded(const DeviceInfo &deviceInfo);
    void deviceInfoRemoved(const DeviceInfo &deviceInfo);


};

#endif // MAVEOSCANNERDEVICEWATCHER_H
