/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
*
* Copyright 2013 - 2020, nymea GmbH
* Contact: contact@nymea.io
*
* This file is part of maveo-scanner.
* This project including source code and documentation is protected by
* copyright law, and remains the property of nymea GmbH. All rights, including
* reproduction, publication, editing and translation, are reserved. The use of
* this project is subject to the terms of a license agreement to be concluded
* with nymea GmbH in accordance with the terms of use of nymea GmbH, available
* under https://nymea.io/license
*
* For any further details and any questions please contact us under
* contact@nymea.io or see our FAQ/Licensing Information on
* https://nymea.io/license/faq
*
* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

#ifndef MAVEOSCANNERINTERFACE_H
#define MAVEOSCANNERINTERFACE_H

#include <QObject>
#include <QSerialPort>
#include <QSerialPortInfo>

#include "maveoscannerutils.h"
#include "maveoscannerpackage.h"
#include "maveoscannerdevicewatcher.h"

class LIBMAVEO_SCANNER_EXPORT MaveoScannerInterface : public QObject
{
    Q_OBJECT
public:
    enum WatchMode {
        WatchModePort,
        WatchModeSerialNumber
    };
    Q_ENUM(WatchMode)

    explicit MaveoScannerInterface(MaveoScannerDeviceWatcher *deviceWatcher, QObject *parent = nullptr);
    ~MaveoScannerInterface();

    WatchMode watchMode() const;
    void setWatchMode(WatchMode watchMode);

    QSerialPort *serialPort() const;

    QString portName() const;
    void setPortName(const QString &portName);

    QString serialNumber() const;
    void setSerialNumber(const QString &serialNumber);

    QSerialPortInfo portInfo() const;

    QSerialPort::BaudRate baudrate() const;
    void setBaudRate(QSerialPort::BaudRate baudrate);

    bool connected() const;
    void sendPackage(const MaveoScannerPackage &package);

    void reconnectInterface();

private:
    WatchMode m_watchMode = WatchModePort;

    MaveoScannerDeviceWatcher *m_deviceWatcher = nullptr;
    QSerialPort *m_serialPort = nullptr;
    bool m_connected = false;
    bool m_enabled = false;

    QString m_portName;
    QString m_serialNumber;
    QSerialPort::BaudRate m_baudrate = QSerialPort::Baud115200;
    QSerialPortInfo searchSerialPortInfo();

    QByteArray m_dataBuffer;
    bool m_escape = false;

    void streamByte(quint8 byte, bool specialCharacter = false);

    void setConnected(bool connected);

    void processData(const QByteArray &data);
    void resetBuffer();

signals:
    void connectedChanged(bool connected);
    void portNameChanged(const QString &portName);
    void packageReceived(const MaveoScannerPackage &package);
    void dataReceived(const QByteArray &data);

private slots:
    bool connectInternally();

    void onReadyRead();
    void onError(QSerialPort::SerialPortError error);

    void onDeviceAdded(const QString &devicePath);
    void onDeviceRemoved(const QString &devicePath);

public slots:
    bool enable();
    void disable();
};

Q_DECLARE_METATYPE(QSerialPort::SerialPortError)

#endif // MAVEOSCANNERINTERFACE_H
