/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
*
* Copyright 2013 - 2020, nymea GmbH
* Contact: contact@nymea.io
*
* This file is part of maveo-scanner.
* This project including source code and documentation is protected by
* copyright law, and remains the property of nymea GmbH. All rights, including
* reproduction, publication, editing and translation, are reserved. The use of
* this project is subject to the terms of a license agreement to be concluded
* with nymea GmbH in accordance with the terms of use of nymea GmbH, available
* under https://nymea.io/license
*
* For any further details and any questions please contact us under
* contact@nymea.io or see our FAQ/Licensing Information on
* https://nymea.io/license/faq
*
* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

#ifndef MAVEOSCANNERPACKAGE_H
#define MAVEOSCANNERPACKAGE_H

#include <QByteArray>

#include "maveoscannerutils.h"

class LIBMAVEO_SCANNER_EXPORT MaveoScannerPackage
{
public:

    enum Type {
        TypeRequest,
        TypeResponse,
        TypeNotification
    };

    MaveoScannerPackage(Type type = TypeRequest);
    MaveoScannerPackage(const QByteArray &packageData, Type type = TypeResponse);

    Type type() const;
    void setType(Type type);

    bool isValid() const;

    MaveoScannerUtils::Target target() const;
    void setTarget(MaveoScannerUtils::Target target);

    MaveoScannerUtils::Error error() const;

    quint8 messageId() const;
    void setMessageId(quint8 messageId);

    QByteArray data() const;
    void setData(const QByteArray data);

    QByteArray packageData() const;
    void setPackageData(const QByteArray &packageData);

    void buildPackage();

private:
    QByteArray m_packageData;
    Type m_type = TypeRequest;

    MaveoScannerUtils::Target m_target = MaveoScannerUtils::TargetUnknown;
    quint8 m_messageId = 0;
    QByteArray m_data;

    bool m_validated = false;

    void parsePackageData();
};

QDebug operator<<(QDebug debug, const MaveoScannerPackage &package);

#endif // MAVEOSCANNERPACKAGE_H
