/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
*
* Copyright 2013 - 2020, nymea GmbH
* Contact: contact@nymea.io
*
* This file is part of maveo-scanner.
* This project including source code and documentation is protected by
* copyright law, and remains the property of nymea GmbH. All rights, including
* reproduction, publication, editing and translation, are reserved. The use of
* this project is subject to the terms of a license agreement to be concluded
* with nymea GmbH in accordance with the terms of use of nymea GmbH, available
* under https://nymea.io/license
*
* For any further details and any questions please contact us under
* contact@nymea.io or see our FAQ/Licensing Information on
* https://nymea.io/license/faq
*
* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

#ifndef MAVEOSCANNERREPLY_H
#define MAVEOSCANNERREPLY_H

#include <QTimer>
#include <QObject>

#include "maveoscannerpackage.h"

class LIBMAVEO_SCANNER_EXPORT MaveoScannerReply : public QObject
{
    Q_OBJECT
    friend class MaveoScanner;

public:
    quint8 messageId() const;
    MaveoScannerUtils::Error error() const;
    QByteArray responseData() const;

    bool aborted() const;
    bool timedOut() const;

private:
    explicit MaveoScannerReply(quint8 messageId, QObject *parent = nullptr);

    quint8 m_messageId = 0;
    MaveoScannerPackage m_requestPackage;
    MaveoScannerPackage m_responsePackage;

    bool m_aborted = false;

    QTimer m_timer;
    bool m_timedOut = false;
    int m_timeoutInterval = 8000;

    void abort();
    void startWait();
    void setFinished();

signals:
    void finished();

private slots:
    void onTimeout();

};

#endif // MAVEOSCANNERREPLY_H
