#ifndef MAVEOSCANNERUTILS_H
#define MAVEOSCANNERUTILS_H

#include <QObject>
#include <QtCore/QtGlobal>
#include <QLoggingCategory>

#if defined(LIBMAVEO_SCANNER)
#    define LIBMAVEO_SCANNER_EXPORT Q_DECL_EXPORT
#else
#    define LIBMAVEO_SCANNER_EXPORT Q_DECL_IMPORT
#endif

Q_DECLARE_LOGGING_CATEGORY(dcScanner)
Q_DECLARE_LOGGING_CATEGORY(dcScannerTraffic)
Q_DECLARE_LOGGING_CATEGORY(dcHoneywellScanner)
Q_DECLARE_LOGGING_CATEGORY(dcHoneywellScannerTraffic)
Q_DECLARE_LOGGING_CATEGORY(dcScannerInterface)
Q_DECLARE_LOGGING_CATEGORY(dcScannerInterfaceTraffic)
Q_DECLARE_LOGGING_CATEGORY(dcScannerDeviceWatcher)
Q_DECLARE_LOGGING_CATEGORY(dcScannerBootloader)
Q_DECLARE_LOGGING_CATEGORY(dcScannerBootloaderTraffic)


#define FTDI_USB_VENDOR  0x0403
#define FTDI_USB_PRODUCT  0x6001


class LIBMAVEO_SCANNER_EXPORT MaveoScannerUtils
{
    Q_GADGET

public:
    enum Target {
        TargetController = 0x10,
        TargetControllerNotification = 0x11,
        TargetLed = 0x12,
        TargetBuzzer = 0x13,
        TargetRelay = 0x14,
        TargetScanner = 0x15,
        TargetScannerNotification = 0x16,
        TargetUnknown = 0xFF
    };
    Q_ENUM(Target)

    enum ControllerCommand {
        ControllerCommandGetFirmware = 0x10,
        ControllerCommandSoftReset = 0x11,
        ControllerCommandHeartbeat = 0x12,
        ControllerCommandGoodRead = 0x13,
        ControllerCommandBadRead = 0x14,
        ControllerCommandStartBootloader = 0x42,
        ControllerCommandGetStatus = 0x50, // Since 0.7.0
        ControllerCommandPlayTestSound = 0x60 // Since 0.7.0
    };
    Q_ENUM(ControllerCommand)

    enum ControllerNotitifactionType {
        ControllerNotitifactionTypeReady = 0x10
    };
    Q_ENUM(ControllerNotitifactionType)


    enum LedCommand {
        LedCommandSetColor = 0x10,
        LedCommandBlink = 0x11,
        LedCommandFlash = 0x12,
        LedCommandDoubleFlash = 0x13
    };
    Q_ENUM(LedCommand)

    enum BuzzerCommand {
        BuzzerCommandPlaySound = 0x10,
        BuzzerCommandPlaySoundSequenceOnce = 0x11,
        BuzzerCommandPlaySoundSequenceForever = 0x12
    };
    Q_ENUM(BuzzerCommand)

    /* Buzzer volume steps */
    enum Volume {
        Volume0         = 0x00u,
        Volume20        = 0x01u,
        Volume40        = 0x02u,
        Volume60        = 0x03u,
        Volume80        = 0x04u,
        Volume100       = 0x05u,
        VolumeInvalid   = 0xFFu
    };
    Q_ENUM(Volume)

    /* Represents a note within an octave */
    enum Note {
        NoteC           = 0x00u,
        NoteCSharp      = 0x01u,
        NoteD           = 0x02u,
        NoteDSharp      = 0x03u,
        NoteE           = 0x04u,
        NoteF           = 0x05u,
        NoteFSharp      = 0x06u,
        NoteG           = 0x07u,
        NoteGSharp      = 0x08u,
        NoteA           = 0x09u,
        NoteASharp      = 0x0Au,
        NoteB           = 0x0Bu,
        NoteInvalid     = 0xFFu
    };
    Q_ENUM(Note)

    /* Represents the available octaves for the buzzer */
    enum Octave {
        Octave1         = 0x00u,
        Octave2         = 0x01u,
        Octave3         = 0x02u,
        Octave4         = 0x03u,
        Octave5         = 0x04u,
        Octave6         = 0x05u,
        Octave7         = 0x06u,
        Octave8         = 0x07u,
        OctaveInvalid   = 0xFFu
    };
    Q_ENUM(Octave)

    enum RelayCommand {
        RelayCommandGetState = 0x10,
        RelayCommandSetState = 0x11
    };
    Q_ENUM(RelayCommand)

    enum ScannerNotification {
        ScannerNotificationDataReceived = 0x10,
        ScannerNotificationErrorOccurred = 0x11
    };
    Q_ENUM(ScannerNotification)

    enum Error {
        ErrorNoError = 0x10,
        ErrorInvalidProtocol = 0x11,
        ErrorUnknownTarget = 0x12,
        ErrorUnknownCommand = 0x13,
        ErrorInvalidMessageFormat = 0x14,
        ErrorBufferOverflow = 0x15,
        ErrorInvalidOctave = 0x16,
        ErrorInvalidNote = 0x17,
        ErrorInvalidVolume = 0x18,
        ErrorInvalidRelayValue = 0x19,
        ErrorTargetNotWritable = 0x1A,
        ErrorScannerBufferOverflow = 0x1B,
        ErrorUnknwonError = 0xFF
    };
    Q_ENUM(Error)

};

#endif // MAVEOSCANNERUTILS_H
